﻿using System;
using System.Configuration;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;
using System.Collections.Generic;

namespace CustomZoomLevelSet
{
    public partial class TestForm : System.Web.UI.Page
    {
        private Proj4Projection baseMapProjection = new Proj4Projection();

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                this.ConfigureMap();
                this.AddSampleMarker();
            }
        }

        private void AddSampleMarker()
        {
            // Get the Lift
            // TODO: This a temp method - delete

            InMemoryMarkerOverlay liftOverlay = (InMemoryMarkerOverlay)this.Map1.CustomOverlays["Lifts"];
            float latitude = 52.653255130651f;
            float longitude = -8.586136783894f;
            string popUpText = "Pop up Text to be added";
            for (int i = 1; i <= 20; i++)
            {
                latitude = latitude + .05f;
                liftOverlay.Features.Add("Lift" + i.ToString(), new Feature(longitude, latitude, i.ToString(), new string[] { "PopUpText:" + popUpText }));
            }
        }
        /// <summary>
        /// Creates the Lift map layer.
        /// </summary>
        private void CreateLiftLayer()
        {
            // Define the columns that will contain data on the features
            List<FeatureSourceColumn> featureSourceColumns = new List<FeatureSourceColumn>();
            featureSourceColumns.Add(new FeatureSourceColumn("PopUpText", "string", 100));

            // Create the overlay
            InMemoryMarkerOverlay liftOverlay = new InMemoryMarkerOverlay("Lifts", featureSourceColumns);

            // Define the Marker General Properties
            // TODO: Add the popup to CSS
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.HasCloseButton = true;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.AutoSize = true;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.BackgroundColor = GeoColor.StandardColors.LightBlue;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.BorderColor = GeoColor.StandardColors.Black;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.BorderWidth = 1;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.IsVisible = false;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.AutoPan = true;

            // Define the Marker image
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.WebImage.ImageOffsetX = -10.5f;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.WebImage.ImageOffsetY = -25f;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.WebImage.ImageWidth = 21;
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.WebImage.ImageHeight = 25;

            // TODO: Add should the icon be in CSS
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.WebImage = new WebImage("theme/lift_map.png", 16, 16);

            // Specify which Feature Source Column will contain the pop up text
            liftOverlay.ZoomLevelSet.ZoomLevel01.DefaultMarkerStyle.Popup.ContentHtml = "[#PopUpText#]";

            // Make it available all over the zoomlevels.
            liftOverlay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Cluster icons at high zoom levels. This prevents points being displayed and bunched together unnecessarily
            // liftOverlay.SuppressingGridSize = 50;

            // TODO: Add this to settings
            liftOverlay.IsVisibleInOverlaySwitcher = true;

            liftOverlay.FeatureSource.Projection = this.baseMapProjection;
            this.Map1.CustomOverlays.Add(liftOverlay);
        }

        /// <summary>
        /// Configures the map.
        /// </summary>
        private void ConfigureMap()
        {
            // TODO: Read the settings from an XML file or from the DB
            this.Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            this.Map1.MapTools.MouseCoordinate.Enabled = true;
            this.Map1.MapTools.MouseCoordinate.MouseCoordinateType = MouseCoordinateType.LatitudeLongitude;
            OverlaySwitcherMapTool overLaySwitcher = new OverlaySwitcherMapTool();

            if (ConfigurationManager.AppSettings["GoogleMapOverlay"].ToString().ToLower() == "true")
            {
                this.Map1.MapUnit = GeographyUnit.Meter;
                this.CreateBaseStaticLayerFromGoogle();

                // Sets the projection parameters to go from Geodetic (EPSG 4326) or decimal degrees to Google Map projection (Spherical Mercator).
                this.baseMapProjection.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
                this.baseMapProjection.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();

                // Set the initial Zoom
                this.Map1.CurrentExtent = this.GoogleMapInitialZoom();

                this.Map1.MapTools.MiniMap.Enabled = false;
                this.Map1.MapTools.ScaleLine.Enabled = false;
                this.Map1.MapTools.OverlaySwitcher.Enabled = false;
            }
            else
            {
                this.Map1.MapUnit = GeographyUnit.DecimalDegree;
                this.Map1.MapTools.MiniMap.Enabled = true;
                this.Map1.MapTools.ScaleLine.Enabled = true;

                overLaySwitcher.Enabled = true;
                this.Map1.MapTools.OverlaySwitcher.Enabled = true;
            }

            this.AddContextMenus();
            this.CreateLiftLayer();
        }

        private RectangleShape GoogleMapInitialZoom()
        {
            // TODO: Move to central settings
            double minx = -11.0; // Mininum Longitude
            double miny = 50.4; // Minimum Latitude
            double maxx = -5.2; // Maximum Longitude
            double maxy = 54; // Maximum Latitude

            // Gets the RectangleShape in the Google Map projection.
            this.baseMapProjection.Open();
            RectangleShape googleMapRectangleShape = this.baseMapProjection.ConvertToExternalProjection(new RectangleShape(minx, maxy, maxx, miny));
            this.baseMapProjection.Close();

            // Sets the extent of the map according to the projected RectangleShape.
            // Notice that a snap to closest zoom level will occur. The map will be centered based on that RectangleShape
            // and it will snap to the closest zoom level to fit the RectangleShape.
            return googleMapRectangleShape;
        }

        private void ContextMenuCentreMapHere_Click(object sender, ContextMenuItemClickEventArgs e)
        {
            // Research http://gis.thinkgeo.com/Support/DiscussionForums/tabid/143/aff/12/aft/5491/afv/topic/Default.aspx#8035
            // for client side functionality
            Map1.CenterAt((double)e.Location.X, (double)e.Location.Y);
        }


        private void AddContextMenus()
        {
            ContextMenu menuOnMap = new ContextMenu("area", 180);

            // TODO: Only execute for GoogleMaps
            ContextMenuItem contextMenuView = new ContextMenuItem("View");

            ContextMenuItem contextMenuViewNormal = new ContextMenuItem("Nornal");
            ContextMenuItem contextMenuViewPhysical = new ContextMenuItem("Physical");
            ContextMenuItem contextMenuViewSatellite = new ContextMenuItem("Satellite");
            ContextMenuItem contextMenuViewHybrid = new ContextMenuItem("Hybrid");

            ContextMenuItem contextMenuZoomIn = new ContextMenuItem("Zoom In");
            ContextMenuItem contextMenuZoomOut = new ContextMenuItem("Zoom Out");
            ContextMenuItem contextMenuCentreMapHere = new ContextMenuItem("Centre Here");

            contextMenuCentreMapHere.Click += new EventHandler<ContextMenuItemClickEventArgs>(this.ContextMenuCentreMapHere_Click);

            // Note : Had to use 4.5.17 Daily Development Build to fix a bug in the context menu
            contextMenuView.MenuItems.Add(contextMenuViewNormal);
            contextMenuView.MenuItems.Add(contextMenuViewPhysical);
            contextMenuView.MenuItems.Add(contextMenuViewSatellite);
            contextMenuView.MenuItems.Add(contextMenuViewHybrid);

            menuOnMap.MenuItems.Add(contextMenuView);
            menuOnMap.MenuItems.Add(contextMenuZoomIn);
            menuOnMap.MenuItems.Add(contextMenuZoomOut);
            menuOnMap.MenuItems.Add(contextMenuCentreMapHere);

            Map1.ContextMenu = menuOnMap;
        }

        /// <summary>
        /// Creates the base static layer from google.
        /// </summary>
        private void CreateBaseStaticLayerFromGoogle()
        {
            GoogleOverlay google = new GoogleOverlay("Google Map");
            google.JavaScriptLibraryUri = new Uri(ConfigurationManager.AppSettings["GoogleUri"]);
            google.GoogleMapType = GoogleMapType.Normal;
            this.Map1.CustomOverlays.Add(google);
        }
    }
}
